
import validators
from termcolor import colored
import re





def check_length(property, min_length, max_length):
  if len(property) >= min_length and len(property) <= max_length:
    return True
  return False

def is_in_list(item, list):
  if item in list:
    return True
  return False

def regex_match(pattern, string):
  pattern = re.compile(pattern)
  if re.fullmatch(pattern, string):
    return True
  return False


def validate_documentation(documentation):

  not_available = 'NA'
  is_valid = {}

  for k,v in documentation.items():

    if k == 'description':
      result = isinstance(v, str) and check_length(v, 2, 3000)
      if result:
        is_valid[k] = True
      else:
        is_valid[k] = False
    else:
      is_valid['description'] = not_available


    if k == 'associatedMedia':
      result = isinstance(v, list) and validators.url(v[0])
      if result:
        is_valid[k] = True
      else:
        is_valid[k] = False
    else:
      is_valid['associatedMedia'] = not_available



    if k == 'isPartOf':
      result = isinstance(v, str) and check_length(v, 2, 80)
      if result:
        is_valid[k] = True
      else:
        is_valid[k] = False
    else:
      is_valid['isPartOf'] = not_available


  return is_valid
    
def check_documentation(documentation, schema_df):
  

  is_valid = validate_documentation(documentation)

  # print(u'\u2713')
  return is_valid
      # if isinstance(v, str) and check_length(v, 2, 80):
        # result = True




import re
import validators
from validate_email import validate_email




def check_length(property, min_length, max_length):
  if len(property) >= min_length and len(property) <= max_length:
    return True
  return False

def is_in_list(item, list):
  if item in list:
    return True
  return False

def regex_match(pattern, string):
  pattern = re.compile(pattern)
  if re.fullmatch(pattern, string):
    return True
  return False


def validate_summary_title(title, schema_df):
    # title_min_length = schema_df['definitions.eightyCharacters.minLength'].iloc[0]
    # title_type = schema_df['definitions.eightyCharacters.type'].iloc[0]
    # title_max_length = schema_df['definitions.eightyCharacters.maxLength'].iloc[0]
    if isinstance(title, str) and check_length(title, 2, 80):
        return True
    return False


  
def validate_summary_abstract(df, schema_df): #row
  abstract = df['summary'].iloc[0]['abstract']
  # title = row['title']
  abstract_min_length = schema_df['definitions.abstractText.minLength'].iloc[0]
  abstract_type = schema_df['definitions.abstractText.type'].iloc[0]
  abstract_max_length = schema_df['definitions.abstractText.maxLength'].iloc[0]
  if isinstance(abstract, str) and check_length(abstract, abstract_min_length, abstract_max_length):
    return True
  return False


def validate_summary_publisher(publisher, schema_df):

  for k,v in publisher.items():
    if k in ['identifier', 'logo', 'accessRights']:
      result = validators.url(v)
    if k == 'name':
      result = isinstance(v, str) and check_length(v, 2, 80)
    if k == 'description':
      result = isinstance(v, str) and check_length(v, 2, 3000)
    if k == 'contactPoint':
      result = validate_email(v) # validate_email('example@example.com',verify=True)

    if k == 'memberOf':
      memberOfList = schema_df['definitions.memberOf.enum']
      result = is_in_list(v, memberOfList)
    if k == 'deliveryLeadTime':
      deliveryLeadTimeList = schema_df['definitions.deliveryLeadTime.enum']
      result = is_in_list(v, deliveryLeadTimeList)
    if k == 'accessService':
      result = isinstance(v, str) and check_length(v, 2, 5000)

    if k == 'accessRequestCost':
      result = isinstance(v, str) and check_length(v, 2, 1000)

    if k in ['dataUseLimitation', 'dataUseRequirements']:
      pattern = r'([^,]+)'
      result = regex_match(pattern, v)

def validate_summary_keywords(keywords, schema_df):

    # keywords = df['summary'].iloc[2]['keywords']

    if isinstance(keywords, list):
      return True

    if isinstance(keywords, str):
      pattern = r'([^,]+)'
      result = regex_match(pattern, keywords)
      if result:
        return True


def validate_summary_alternateIdentifiers(alternateIdentifiers, schema_df):

    # alternateIdentifiers = df['summary'].iloc[2]['alternateIdentifiers']

    if isinstance(alternateIdentifiers, list):
      return True

    if isinstance(alternateIdentifiers, str):
      pattern = r'([^,]+)'
      result = regex_match(pattern, alternateIdentifiers)
      if result:
        return True


def validate_summary_doiName(doiName, schema_df):

    # alternateIdentifiers = df['summary'].iloc[2]['alternateIdentifiers']

    if isinstance(doiName, str):
      pattern = r'^10.\d{4,9}/[-._;()/:a-zA-Z0-9]+$'
      result = regex_match(pattern, doiName)
      if result:
        return True



def validate_summary(summary, schema_df):

    for k,v in summary.items():

      if k == 'title':
        title = summary['title']
        validate_summary_title(title, schema_df)

      if k == 'abstract':
        abstract = summary['abstract']
        validate_summary_abstract(abstract, schema_df)


    if k == 'publisher':
      publisher = summary['publisher']

      validate_summary_publisher(publisher, schema_df)


    if k == 'contactPoint':
        contactPoint = summary['contactPoint']
        is_valid = validate_email(contactPoint)

    #   validate_summary_contactPoint(contactPoint)


    if k == 'keywords':
      keywords = summary['keywords']

      validate_summary_keywords(keywords, schema_df)


    if k == 'alternateIdentifiers':
      alternateIdentifiers = summary['alternateIdentifiers']

      validate_summary_alternateIdentifiers(alternateIdentifiers, schema_df)

    if k == 'doiName':
      doiName = summary['doiName']

      validate_summary_doiName(doiName)



    
def check_summary(df, schema_df):

  summary = df['summary'].iloc[2]['summary']

  validate_summary(summary)






# def check_summary_publisher(df, schema_df):

#   publisher = df['summary'].iloc[2]['publisher']

#   validate_publisher(publisher)

      # if isinstance(v, str) and check_length(v, 2, 80):
        # result = True




import re
import validators
from validate_email import validate_email




def check_length(property, min_length, max_length):
  if len(property) >= min_length and len(property) <= max_length:
    return True
  return False

def is_in_list(item, list):
  if item in list:
    return True
  return False

def regex_match(pattern, string):
  pattern = re.compile(pattern)
  if re.fullmatch(pattern, string):
    return True
  return False


def validate_observations_observedNode(observedNode, schema_df):
    # title_min_length = schema_df['definitions.eightyCharacters.minLength'].iloc[0]
    # title_type = schema_df['definitions.eightyCharacters.type'].iloc[0]
    # title_max_length = schema_df['definitions.eightyCharacters.maxLength'].iloc[0]
    if isinstance(observedNode, str) and check_length(observedNode, 2, 80):
        return True
    return False


  
def validate_observations_measuredValue(df, schema_df): #row
  measuredValue = df['observations'].iloc[0]['measuredValue']
  # title = row['title']
  measuredValue_min_length = schema_df['definitions.measuredValueText.minLength'].iloc[0]
  measuredValue_type = schema_df['definitions.measuredValueText.type'].iloc[0]
  measuredValue_max_length = schema_df['definitions.measuredValueText.maxLength'].iloc[0]
  if isinstance(measuredValue, str) and check_length(measuredValue, measuredValue_min_length, measuredValue_max_length):
    return True
  return False


def validate_observations_disambiguatingDescription(disambiguatingDescription, schema_df):

  for k,v in disambiguatingDescription.items():
    if k in ['identifier', 'logo', 'accessRights']:
      result = validators.url(v)
    if k == 'name':
      result = isinstance(v, str) and check_length(v, 2, 80)
    if k == 'description':
      result = isinstance(v, str) and check_length(v, 2, 3000)
    if k == 'observationDate':
      result = validate_email(v) # validate_email('example@example.com',verify=True)

    if k == 'memberOf':
      memberOfList = schema_df['definitions.memberOf.enum']
      result = is_in_list(v, memberOfList)
    if k == 'deliveryLeadTime':
      deliveryLeadTimeList = schema_df['definitions.deliveryLeadTime.enum']
      result = is_in_list(v, deliveryLeadTimeList)
    if k == 'accessService':
      result = isinstance(v, str) and check_length(v, 2, 5000)

    if k == 'accessRequestCost':
      result = isinstance(v, str) and check_length(v, 2, 1000)

    if k in ['dataUseLimitation', 'dataUseRequirements']:
      pattern = r'([^,]+)'
      result = regex_match(pattern, v)

def validate_observations_measuredProperty(measuredProperty, schema_df):

    # measuredProperty = df['observations'].iloc[2]['measuredProperty']

    if isinstance(measuredProperty, list):
      return True

    if isinstance(measuredProperty, str):
      pattern = r'([^,]+)'
      result = regex_match(pattern, measuredProperty)
      if result:
        return True


# def validate_observations_alternateIdentifiers(alternateIdentifiers, schema_df):

#     # alternateIdentifiers = df['observations'].iloc[2]['alternateIdentifiers']

#     if isinstance(alternateIdentifiers, list):
#       return True

#     if isinstance(alternateIdentifiers, str):
#       pattern = r'([^,]+)'
#       result = regex_match(pattern, alternateIdentifiers)
#       if result:
#         return True


# def validate_observations_doiName(doiName, schema_df):

#     # alternateIdentifiers = df['observations'].iloc[2]['alternateIdentifiers']

#     if isinstance(doiName, str):
#       pattern = r'^10.\d{4,9}/[-._;()/:a-zA-Z0-9]+$'
#       result = regex_match(pattern, doiName)
#       if result:
#         return True



def validate_observations(observations, schema_df):

    for k,v in observations.items():

      if k == 'observedNode':
        observedNode = observations['observedNode']
        observedNodeList = schema_df['definitions.statisticalPopulationConstrained.enum'][0]
        return observedNode in observedNodeList
        # validate_observations_observedNode(observedNode, schema_df)

      if k == 'measuredValue':
        measuredValue = observations['measuredValue']
        return isinstance(disambiguatingDescription, str)
        # df['observations'].iloc[2][0]['measuredValue']

        # validate_observations_measuredValue(measuredValue, schema_df)


    if k == 'disambiguatingDescription':
        disambiguatingDescription = observations['disambiguatingDescription']
        result = isinstance(disambiguatingDescription, str)

    #   validate_observations_disambiguatingDescription(disambiguatingDescription, schema_df)


    if k == 'observationDate':
        observationDate = observations['observationDate']
        result = isinstance(observationDate, str)
        # is_valid = validate_email(observationDate)

    #   validate_observations_observationDate(observationDate)


    if k == 'measuredProperty':
        measuredProperty = observations['measuredProperty']
        measuredPropertyList = schema_df['definitions.observation.properties.measuredProperty.allOf'][0][0]['enum']
        return measuredProperty in measuredPropertyList
        # validate_observations_measuredProperty(measuredProperty, schema_df)










    # if k == 'alternateIdentifiers':
    #   alternateIdentifiers = observations['alternateIdentifiers']

    #   validate_observations_alternateIdentifiers(alternateIdentifiers, schema_df)

    # if k == 'doiName':
    #   doiName = observations['doiName']

    #   validate_observations_doiName(doiName)



    
def check_observations(df, schema_df):

  observations = df['observations'].iloc[2][0]

  validate_observations(observations, schema_df)






# def check_observations_disambiguatingDescription(df, schema_df):

#   disambiguatingDescription = df['observations'].iloc[2]['disambiguatingDescription']

#   validate_disambiguatingDescription(disambiguatingDescription)

      # if isinstance(v, str) and check_length(v, 2, 80):
        # result = True










from operator import not_
import validators
import re


def check_length(property, min_length, max_length):
  if len(property) >= min_length and len(property) <= max_length:
    return True
  return False

def is_in_list(item, list):
  if item in list:
    return True
  return False

def regex_match(pattern, string):
  pattern = re.compile(pattern)
  if re.fullmatch(pattern, string):
    return True
  return False




# def save_result(result, k):
#       if result:
#         is_valid[k] = True
#       else:
#         is_valid[k] = False
#   else:
#     is_valid[k] = not_available







def vaildated_accessibility_usage(usage):
  


  print('Checking accessibility usage')

  not_available = 'NA'
  is_valid = {}

  check_list = ['dataUseLimitation', 'dataUseRequirements', 'resourceCreator', 'investigations', 'isReferencedBy']
  keys = list(usage.keys())
  na = set(check_list) - set(keys)

  for i in na:
    is_valid[i] = not_available


  for k,v in usage.items():

      if k == 'dataUseLimitation':
        print('Checking dataUseLimitation')
        print('v list: ', v)
        pattern = r'([^,]+)'

        if isinstance(v, str): 
          result = regex_match(pattern, v)
          if result:
            is_valid[k] = True
          else:
            is_valid[k] = False
        else:
          is_valid[k] = not_available


        if isinstance(v, list) and len(v) > 0:
          result = regex_match(pattern, v[0])
        # if isinstance(v, str): 
        #   result = regex_match(pattern, v)
          if result:
            is_valid[k] = True
          else:
            is_valid[k] = False
        else:
          is_valid[k] = not_available




      if k == ' dataUseRequirements':
        print('dataUseRequirements')
        pattern = r'([^,]+)'
        if isinstance(v, str): 
          result = regex_match(pattern, v)
          if result:
            is_valid[k] = True
          else:
            is_valid[k] = False
        else:
          is_valid[k] = not_available


        if isinstance(v, list):
          result = regex_match(pattern, v[0])
          if result:
            is_valid[k] = True
          else:
            is_valid[k] = False
        else:
          is_valid[k] = not_available



      if k == 'resourceCreator':
        print('resourceCreator')
        if isinstance(v, str):
          result =  check_length(v, 2, 1000)
          if result:
            is_valid[k] = True
          else:
            is_valid[k] = False
        else:
          is_valid[k] = not_available



        if isinstance(v, list):
          result =  check_length(v, 2, 1000)
          if result:
            is_valid[k] = True
          else:
            is_valid[k] = False
        else:
          is_valid[k] = not_available





      if k == 'investigations':
        print('investigations')
        print('v: ', v)
        pattern = r'([^,]+)'
        if isinstance(v, str): 
          result = regex_match(pattern, v)
          if result:
            is_valid[k] = True
          else:
            is_valid[k] = False
        else:
          is_valid[k] = not_available

        if isinstance(v, list):
          result = validators.url(v[0])
          if result:
            is_valid[k] = True
          else:
            is_valid[k] = False
        else:
          is_valid[k] = not_available


      if k == 'isReferencedBy':
        print('isReferencedBy')
        pattern = r'^10.\\d{4,9}/[-._;()/:a-zA-Z0-9]+$'
        if isinstance(v, str): 
          result = regex_match(pattern, v)
          if result:
            is_valid[k] = True
          else:
            is_valid[k] = False
        else:
          is_valid[k] = not_available

        if isinstance(v, list):
          result = regex_match(pattern, v[0])
          if result:
            is_valid[k] = True
          else:
            is_valid[k] = False
        else:
          is_valid[k] = not_available
  

  return is_valid





def validate_accessibility_access(access, schema_df):

      print('Checking accessibility access')

      not_available = 'NA'
      is_valid = {}

      check_list = ['accessRights', 'accessService', 'accessRequestCost', 'deliveryLeadTime', 'jurisdiction', 'dataController', 'dataProcessor']
      keys = list(access.keys())
      na = set(check_list) - set(keys)

      for i in na:
        is_valid[i] = not_available

      for k,v in access.items():

        if k == 'accessRights':
          print('Access rights')
          if isinstance(v, str): 
            result =  validators.url(v)
            if result:
              is_valid[k] = True
            else:
              is_valid[k] = False
          else:
            is_valid[k] = not_available


          if isinstance(v, list):
            result =  validators.url(v[0])
            if result:
              is_valid[k] = True
            else:
              is_valid[k] = False
          else:
            is_valid[k] = not_available



        if k == 'accessService':
          print('Checking access service')
          result = isinstance(v, str) and check_length(v, 2, 5000)
          if result:
            is_valid[k] = True
          else:
            is_valid[k] = False
        else:
          is_valid[k] = not_available

        if k == 'accessRequestCost':
          print('Access request cost')
          if isinstance(v, str):
            result =  check_length(v, 2, 5000)
            if result:
              is_valid[k] = True
            else:
              is_valid[k] = False
          else:
            is_valid[k] = not_available

          if isinstance(v, list):
            result = validators.url(v[0])
            if result:
              is_valid[k] = True
            else:
              is_valid[k] = False
          else:
            is_valid[k] = not_available



        if k == 'deliveryLeadTime':
          print('deliveryLeadTime')
          deliveryLeadTimeList = schema_df['definitions.deliveryLeadTime.enum']
          result = is_in_list(v, deliveryLeadTimeList)
          if result:
            is_valid[k] = True
          else:
            is_valid[k] = False
        else:
          is_valid[k] = not_available


        if k == 'jurisdiction':
          print('jurisdiction')
          print('v: ', v)
          pattern1 = r'([^,]+)'
          pattern2 = r'^[A-Z]{2}(-[A-Z]{2,3})?$'
          result = regex_match(pattern1, v[0]) #or regex_match(pattern2, v)
          print('result:', result)
          if result:
            is_valid[k] = True
          else:
            is_valid[k] = False
        else:
          is_valid[k] = not_available

        if k in ['dataController', 'dataProcessor']:
          print('dataController')
          result = isinstance(v, str) and check_length(v, 2, 5000)
          if result:
            is_valid[k] = True
          else:
            is_valid[k] = False
        else:
          is_valid[k] = not_available

      return is_valid



def validate_accessibility_formatStandards(formatAndStandards, schema_df):

      print('Checking accessibility formatStandards')
      not_available = 'NA'
      is_valid = {}

      check_list = ['vocabularyEncodingScheme', 'conformsTo', 'language', 'format']
      keys = list(formatAndStandards.keys())
      na = set(check_list) - set(keys)

      for i in na:
        is_valid[i] = not_available


      for k,v in formatAndStandards.items():

        print('k: ', k)
        print('v: ', v)
        if k == 'vocabularyEncodingScheme':
          print('key: vocabularyEncodingScheme')
          if isinstance(v, str): 
            pattern = r'([^,]+)'
            result = regex_match(pattern, v)   
            if result:
              is_valid[k] = True
            else:
              is_valid[k] = False
          else:
            is_valid[k] = not_available

          if isinstance(v, list):
            print('checking list vocabularyEncodingSchemeList')
            vocabularyEncodingSchemeList = schema_df['definitions.controlledVocabulary.enum'].values[0]
            # print('list: ', vocabularyEncodingSchemeList.values)
            result = set(v) <= set(vocabularyEncodingSchemeList)
            # result =  v in vocabularyEncodingSchemeList
            print('here')
            print('result: ', result)

            if result:
              is_valid[k] = True
            else:
              is_valid[k] = False
          else:
            is_valid[k] = not_available


        if k == 'conformsTo':
          if isinstance(v, str): 
            pattern = r'([^,]+)'
            result = regex_match(pattern, v)  
            if result:
              is_valid[k] = True
            else:
              is_valid[k] = False
          else:
            is_valid[k] = not_available

          # if isinstance(v, list):
          #   standardisedDataModelsList = schema_df['definitions.standardisedDataModels.enum']
          #   result =  v in standardisedDataModelsList
          #   if result:
          #     is_valid[k] = True
          #   else:
          #     is_valid[k] = False
          # else:
          #   is_valid[k] = not_available




        if k == 'language':
          if isinstance(v, str): 
            pattern = r'([^,]+)'
            result = regex_match(pattern, v)  
            if result:
              is_valid[k] = True
            else:
              is_valid[k] = False
          else:
            is_valid[k] = not_available

          # if isinstance(v, list):
          #   languageList = schema_df['definitions.language.enum']
          #   result =  v in languageList
          #   if result:
          #     is_valid[k] = True
          #   else:
          #     is_valid[k] = False
          # else:
          #   is_valid[k] = not_available


        if k == 'format':
          if isinstance(v, str) and len(v) > 1: 
            pattern = r'([^,]+)'
            result = regex_match(pattern, v)  
            if result:
              is_valid[k] = True
            else:
              is_valid[k] = False
          else:
            is_valid[k] = not_available


      return is_valid      




def validate_accessibility(accessibility, schema_df):

    is_valid = {}
    not_available = 'NA'
    check_list = ['access', 'usage', 'formatAndStandards']
    keys = list(accessibility.keys())
    na = set(check_list) - set(keys)

    for i in na:
      is_valid[i] = not_available


    for k,v in accessibility.items():

      if k == 'usage':
        usage = accessibility['usage']
        result = vaildated_accessibility_usage(usage)
        is_valid[k] = result


      if k == 'access':
        access = accessibility['access']
        result = validate_accessibility_access(access, schema_df)
        is_valid[k] = result


      if k == 'formatAndStandards':
        formatAndStandards = accessibility['formatAndStandards']

        result = validate_accessibility_formatStandards(formatAndStandards, schema_df)
        is_valid[k] = result
  
    return is_valid



    
def check_accessibility(accessibility, schema_df):

  is_valid = validate_accessibility(accessibility, schema_df)

  return is_valid



# def check_summary_publisher(df, schema_df):

#   publisher = df['summary'].iloc[2]['publisher']

#   validate_publisher(publisher)

      # if isinstance(v, str) and check_length(v, 2, 80):
        # result = True




import re



def regex_match(pattern, string):
  pattern = re.compile(pattern)
  if re.fullmatch(pattern, string):
    return True
  return False


def check_version(version, schema_df):
  version_pattern = schema_df['definitions.semver.pattern'].values[0]
  is_version = regex_match(version_pattern, version)
  return is_version


from .accessibility import check_accessibility
from .documentation import check_documentation
from .identifier import check_identifier
from .revisions import check_revisions
from .summary import check_summary
from .version import check_version
from .issued import check_issued
from .modified import check_modified
from .observations import check_observations
from collections import defaultdict
import pandas as pd
import json
from termcolor import colored



def parse_column(df, col):
    series = df[col]
    df_list = defaultdict(list)
    nrows = series.shape[0]

    for i in range(nrows):
      for k, v in zip(series.iloc[i].keys(), series.iloc[i].values()):
        df_list[k].append(v)
    
    df = pd.DataFrame.from_dict(dict(df_list.items()), orient='index').T
    
    return df



def parse_json_file(metadata_path, data_model_col='dataModels'):

    df_json = pd.read_json(metadata_path)

    data_models = df_json[data_model_col]
    df_list = defaultdict(list)
    nrows = data_models.shape[0]

    for i in range(nrows):
      for k, v in zip(data_models.iloc[i].keys(), data_models.iloc[i].values()):
        df_list[k].append(v)
    
    df = pd.DataFrame.from_dict(dict(df_list.items()), orient='index').T
    
    return df

# df = parse_json_file(metadata_path)

def load_schema(schema_file_path):
    # schema_path = "dataset.schema.json"

    with open(schema_file_path, 'r') as j:
        schema_contents = json.loads(j.read())

    schema_df = pd.json_normalize(schema_contents)

    return schema_df


def validate_property(df, schema_df, col, validator):
        
        print(colored(f'Checking {col} for conformitiy', 'red'))

        result = df[col].apply((lambda x: validator(x, schema_df)))

        print(colored(f"Checking {col} complete", 'green'), colored(u'\u2713', 'green'))

        return result




def schema_validator(metadata_path, schema_file_path):
     df = parse_json_file(metadata_path)
     schema_df = load_schema(schema_file_path)
     checklist = list(df.columns)
     validated_dict = {}

     for col in checklist:
        

        # if col == 'identifier':
        #     result = validate_property(df, schema_df, col, check_identifier)
        #     validated_dict[col] = result


        if col == 'version':
            result = validate_property(df, schema_df, col, check_version)
            validated_dict[col] = result
        
        if col == 'documentation':
            result = validate_property(df, schema_df, col, check_documentation)
            validated_dict[col] = result

        if col == 'accessibility':
            result = validate_property(df, schema_df, col, check_accessibility)
            validated_dict[col] = result        

     return validated_dict












import validators









def check_length(property, min_length, max_length):
  if len(property) >= min_length and len(property) <= max_length:
    return True
  return False

def is_in_list(item, list):
  if item in list:
    return True
  return False

def regex_match(pattern, string):
  pattern = re.compile(pattern)
  if re.fullmatch(pattern, string):
    return True
  return False


def validate_revisions(revisions):

  for k,v in revisions.items():

    if k == 'version':
        pattern = r'^([0-9]+)\.([0-9]+)\.([0-9]+)$'
        result = regex_match(pattern, v)

    if k == 'url':
      result = isinstance(v, str) and validators.url(v)


def check_revisions(df, schema_df):
  
  revisions = df['summary'].iloc[2]['revisions']

  validate_revisions(revisions)
      # if isinstance(v, str) and check_length(v, 2, 80):
        # result = True






import pandas as pd
import validators


def check_identifier(identifier, schema_df):

  identifier_req_1 = schema_df[schema_df.columns[pd.Series(schema_df.columns).str.startswith('properties.identifier')]]['properties.identifier.anyOf'][0][0]['$ref']
  identifier_req_2 = schema_df[schema_df.columns[pd.Series(schema_df.columns).str.startswith('properties.identifier')]]['properties.identifier.anyOf'][0][1]['$ref']
  identifier_def_1 = identifier_req_1.strip('#/').replace('/','.')
  identifier_def_2 = identifier_req_2.strip('#/').replace('/','.')

  identifier_uuid_pattern = schema_df[schema_df.columns[pd.Series(schema_df.columns).str.startswith(identifier_def_1)]]['definitions.uuidv4.pattern']
  identifier_uuid_type = schema_df[schema_df.columns[pd.Series(schema_df.columns).str.startswith(identifier_def_1)]]['definitions.uuidv4.type']
  identifier_uuid_min_length = schema_df[schema_df.columns[pd.Series(schema_df.columns).str.startswith(identifier_def_1)]]['definitions.uuidv4.minLength'].values[0]
  identifier_uuid_max_length = schema_df[schema_df.columns[pd.Series(schema_df.columns).str.startswith(identifier_def_1)]]['definitions.uuidv4.maxLength'].values[0]
  identifier_url_type = schema_df[schema_df.columns[pd.Series(schema_df.columns).str.startswith(identifier_def_2)]]['definitions.url.type']
  identifier_url_format = schema_df[schema_df.columns[pd.Series(schema_df.columns).str.startswith(identifier_def_2)]]['definitions.url.format']

  return validators.url(identifier)

